@@ -22,11 +22,11 @@
 import time
 import collections
 
-from PyQt5.QtCore import pyqtSignal, QUrl, QObject
-from PyQt5.QtWebKit import QWebHistoryInterface
+from PyQt5.QtCore import pyqtSignal, pyqtSlot, QUrl, QObject
 
 from qutebrowser.commands import cmdutils
-from qutebrowser.utils import utils, objreg, standarddir, log, qtutils
+from qutebrowser.utils import (utils, objreg, standarddir, log, qtutils,
+                               usertypes)
 from qutebrowser.config import config
 from qutebrowser.misc import lineparser
 
@@ -88,12 +88,8 @@ class Entry:
         if not url.isValid():
             raise ValueError("Invalid URL: {}".format(url.errorString()))
 
-        if atime.startswith('\0'):
-            log.init.debug(
-                "Removing NUL bytes from entry {!r} - see "
-                "https://github.com/The-Compiler/qutebrowser/issues/"
-                "670".format(data))
-            atime = atime.lstrip('\0')
+        # https://github.com/The-Compiler/qutebrowser/issues/670
+        atime = atime.lstrip('\0')
 
         if '-' in atime:
             atime, flags = atime.split('-')
@@ -108,34 +104,6 @@ class Entry:
         return cls(atime, url, title, redirect=redirect)
 
 
-class WebHistoryInterface(QWebHistoryInterface):
-
-    """Glue code between WebHistory and Qt's QWebHistoryInterface.
-
-    Attributes:
-        _history: The WebHistory object.
-    """
-
-    def __init__(self, webhistory, parent=None):
-        super().__init__(parent)
-        self._history = webhistory
-
-    def addHistoryEntry(self, url_string):
-        """Required for a QWebHistoryInterface impl, obsoleted by add_url."""
-        pass
-
-    def historyContains(self, url_string):
-        """Called by WebKit to determine if an URL is contained in the history.
-
-        Args:
-            url_string: The URL (as string) to check for.
-
-        Return:
-            True if the url is in the history, False otherwise.
-        """
-        return url_string in self._history.history_dict
-
-
 class WebHistory(QObject):
 
     """The global history of visited pages.
@@ -157,7 +125,6 @@ class WebHistory(QObject):
 
     Attributes:
         history_dict: An OrderedDict of URLs read from the on-disk history.
-        _hist_dir: The directory to store the history in
         _lineparser: The AppendLineParser used to save the history.
         _new_history: A list of Entry items of the current session.
         _saved_count: How many HistoryEntries have been written to disk.
@@ -185,7 +152,6 @@ class WebHistory(QObject):
         super().__init__(parent)
         self._initial_read_started = False
         self._initial_read_done = False
-        self._hist_dir = hist_dir
         self._lineparser = lineparser.AppendLineParser(hist_dir, hist_name,
                                                        parent=self)
         self.history_dict = collections.OrderedDict()
@@ -211,12 +177,6 @@ class WebHistory(QObject):
             return
         self._initial_read_started = True
 
-        if self._hist_dir is None:
-            self._initial_read_done = True
-            self.async_read_done.emit()
-            assert not self._temp_history
-            return
-
         with self._lineparser.open():
             for line in self._lineparser:
                 yield
@@ -284,17 +244,32 @@ class WebHistory(QObject):
         self._saved_count = 0
         self.cleared.emit()
 
+    @pyqtSlot(QUrl, QUrl, str)
+    def add_from_tab(self, url, requested_url, title):
+        """Add a new history entry as slot, called from a BrowserTab."""
+        no_formatting = QUrl.UrlFormattingOption(0)
+        if (requested_url.isValid() and
+                not requested_url.matches(url, no_formatting)):
+            # If the url of the page is different than the url of the link
+            # originally clicked, save them both.
+            self.add_url(requested_url, title, redirect=True)
+        self.add_url(url, title)
+
     def add_url(self, url, title="", *, redirect=False, atime=None):
-        """Called by WebKit when an URL should be added to the history.
+        """Called via add_from_tab when a URL should be added to the history.
 
         Args:
-            url: An url (as QUrl) to add to the history.
+            url: A url (as QUrl) to add to the history.
             redirect: Whether the entry was redirected to another URL
                       (hidden in completion)
             atime: Override the atime used to add the entry
         """
         if config.get('general', 'private-browsing'):
             return
+        if not url.isValid():
+            log.misc.warning("Ignoring invalid URL being added to history")
+            return
+
         if atime is None:
             atime = time.time()
         entry = Entry(atime, url, title, redirect=redirect)
@@ -318,5 +293,7 @@ def init(parent=None):
                          parent=parent)
     objreg.register('web-history', history)
 
-    interface = WebHistoryInterface(history, parent=history)
-    QWebHistoryInterface.setDefaultInterface(interface)
+    used_backend = usertypes.arg2backend[objreg.get('args').backend]
+    if used_backend == usertypes.Backend.QtWebKit:
+        from qutebrowser.browser.webkit import webkithistory
+        webkithistory.init(history)
