@@ -24,6 +24,7 @@ are fundamentally different. This is why nothing inherits from configparser,
 but we borrow some methods and classes from there where it makes sense.
 """
 
+import re
 import os
 import sys
 import os.path
@@ -34,9 +35,11 @@ import collections
 import collections.abc
 
 from PyQt5.QtCore import pyqtSignal, QObject, QUrl, QSettings
+from PyQt5.QtGui import QColor
 
 from qutebrowser.config import configdata, configexc, textwrapper
-from qutebrowser.config.parsers import ini, keyconf
+from qutebrowser.config.parsers import keyconf
+from qutebrowser.config.parsers import ini
 from qutebrowser.commands import cmdexc, cmdutils
 from qutebrowser.utils import (message, objreg, utils, standarddir, log,
                                qtutils, error, usertypes)
@@ -157,19 +160,18 @@ def _init_main_config(parent=None):
         sys.exit(usertypes.Exit.err_config)
     else:
         objreg.register('config', config_obj)
-        if standarddir.config() is not None:
-            filename = os.path.join(standarddir.config(), 'qutebrowser.conf')
-            save_manager = objreg.get('save-manager')
-            save_manager.add_saveable(
-                'config', config_obj.save, config_obj.changed,
-                config_opt=('general', 'auto-save-config'), filename=filename)
-            for sect in config_obj.sections.values():
-                for opt in sect.values.values():
-                    if opt.values['conf'] is None:
-                        # Option added to built-in defaults but not in user's
-                        # config yet
-                        save_manager.save('config', explicit=True, force=True)
-                        return
+        filename = os.path.join(standarddir.config(), 'qutebrowser.conf')
+        save_manager = objreg.get('save-manager')
+        save_manager.add_saveable(
+            'config', config_obj.save, config_obj.changed,
+            config_opt=('general', 'auto-save-config'), filename=filename)
+        for sect in config_obj.sections.values():
+            for opt in sect.values.values():
+                if opt.values['conf'] is None:
+                    # Option added to built-in defaults but not in user's
+                    # config yet
+                    save_manager.save('config', explicit=True, force=True)
+                    return
 
 
 def _init_key_config(parent):
@@ -194,13 +196,12 @@ def _init_key_config(parent):
         sys.exit(usertypes.Exit.err_key_config)
     else:
         objreg.register('key-config', key_config)
-        if standarddir.config() is not None:
-            save_manager = objreg.get('save-manager')
-            filename = os.path.join(standarddir.config(), 'keys.conf')
-            save_manager.add_saveable(
-                'key-config', key_config.save, key_config.config_dirty,
-                config_opt=('general', 'auto-save-config'), filename=filename,
-                dirty=key_config.is_dirty)
+        save_manager = objreg.get('save-manager')
+        filename = os.path.join(standarddir.config(), 'keys.conf')
+        save_manager.add_saveable(
+            'key-config', key_config.save, key_config.config_dirty,
+            config_opt=('general', 'auto-save-config'), filename=filename,
+            dirty=key_config.is_dirty)
 
 
 def _init_misc():
@@ -234,10 +235,7 @@ def _init_misc():
     # This fixes one of the corruption issues here:
     # https://github.com/The-Compiler/qutebrowser/issues/515
 
-    if standarddir.config() is None:
-        path = os.devnull
-    else:
-        path = os.path.join(standarddir.config(), 'qsettings')
+    path = os.path.join(standarddir.config(), 'qsettings')
     for fmt in [QSettings.NativeFormat, QSettings.IniFormat]:
         QSettings.setPath(fmt, QSettings.UserScope, path)
 
@@ -285,6 +283,50 @@ def _transform_position(val):
         return val
 
 
+def _transform_hint_color(val):
+    """Transformer for hint colors."""
+    log.config.debug("Transforming hint value {}".format(val))
+
+    def to_rgba(qcolor):
+        """Convert a QColor to a rgba() value."""
+        return 'rgba({}, {}, {}, 0.8)'.format(qcolor.red(), qcolor.green(),
+                                              qcolor.blue())
+
+    if val.startswith('-webkit-gradient'):
+        pattern = re.compile(r'-webkit-gradient\(linear, left top, '
+                             r'left bottom, '
+                             r'color-stop\(0%, *([^)]*)\), '
+                             r'color-stop\(100%, *([^)]*)\)\)')
+
+        match = pattern.fullmatch(val)
+        if match:
+            log.config.debug('Color groups: {}'.format(match.groups()))
+            start_color = QColor(match.group(1))
+            stop_color = QColor(match.group(2))
+            if not start_color.isValid() or not stop_color.isValid():
+                return None
+
+            return ('qlineargradient(x1:0, y1:0, x2:0, y2:1, stop:0 {}, '
+                    'stop:1 {})'.format(to_rgba(start_color),
+                                        to_rgba(stop_color)))
+        else:
+            return None
+    elif val.startswith('-'):  # Custom CSS stuff?
+        return None
+    else:  # Already transformed or a named color.
+        return val
+
+
+def _transform_hint_font(val):
+    """Transformer for fonts -> hints."""
+    match = re.fullmatch(r'(.*\d+p[xt]) Monospace', val)
+    if match:
+        # Close enough to the old default:
+        return match.group(1) + ' ${_monospace}'
+    else:
+        return val
+
+
 class ConfigManager(QObject):
 
     """Configuration manager for qutebrowser.
@@ -338,6 +380,14 @@ class ConfigManager(QObject):
         ('completion', 'history-length'): 'cmd-history-max-items',
         ('colors', 'downloads.fg'): 'downloads.fg.start',
         ('ui', 'show-keyhints'): 'keyhint-blacklist',
+        ('content', 'javascript-can-open-windows'):
+            'javascript-can-open-windows-automatically',
+        ('colors', 'statusbar.fg.error'): 'messages.fg.error',
+        ('colors', 'statusbar.bg.error'): 'messages.bg.error',
+        ('colors', 'statusbar.fg.warning'): 'messages.fg.warning',
+        ('colors', 'statusbar.bg.warning'): 'messages.bg.warning',
+        ('colors', 'statusbar.fg.prompt'): 'prompts.fg',
+        ('colors', 'statusbar.bg.prompt'): 'prompts.bg',
     }
     DELETED_OPTIONS = [
         ('colors', 'tab.separator'),
@@ -350,12 +400,28 @@ class ConfigManager(QObject):
         ('tabs', 'auto-hide'),
         ('tabs', 'hide-always'),
         ('ui', 'display-statusbar-messages'),
+        ('ui', 'hide-mouse-cursor'),
         ('general', 'wrap-search'),
+        ('hints', 'opacity'),
+        ('completion', 'auto-open'),
     ]
     CHANGED_OPTIONS = {
         ('content', 'cookies-accept'):
             _get_value_transformer({'default': 'no-3rdparty'}),
+        ('tabs', 'new-tab-position'):
+            _get_value_transformer({
+                'left': 'prev',
+                'right': 'next'}),
+        ('tabs', 'new-tab-position-explicit'):
+            _get_value_transformer({
+                'left': 'prev',
+                'right': 'next'}),
         ('tabs', 'position'): _transform_position,
+        ('tabs', 'select-on-remove'):
+            _get_value_transformer({
+                'left': 'prev',
+                'right': 'next',
+                'previous': 'last-used'}),
         ('ui', 'downloads-position'): _transform_position,
         ('ui', 'remove-finished-downloads'):
             _get_value_transformer({'false': '-1', 'true': '1000'}),
@@ -363,6 +429,19 @@ class ConfigManager(QObject):
             _get_value_transformer({'false': 'none', 'true': 'debug'}),
         ('ui', 'keyhint-blacklist'):
             _get_value_transformer({'false': '*', 'true': ''}),
+        ('hints', 'auto-follow'):
+            _get_value_transformer({'false': 'never', 'true': 'unique-match'}),
+        ('colors', 'hints.bg'): _transform_hint_color,
+        ('colors', 'hints.fg'): _transform_hint_color,
+        ('colors', 'hints.fg.match'): _transform_hint_color,
+        ('fonts', 'hints'): _transform_hint_font,
+        ('completion', 'show'):
+            _get_value_transformer({'false': 'never', 'true': 'always'}),
+        ('ui', 'user-stylesheet'):
+            _get_value_transformer({
+                'html > ::-webkit-scrollbar { width: 0px; height: 0px; }': '',
+                '::-webkit-scrollbar { width: 0px; height: 0px; }': '',
+            }),
     }
 
     changed = pyqtSignal(str, str)
@@ -419,7 +498,7 @@ class ConfigManager(QObject):
         for optname, option in sect.items():
 
             lines.append('#')
-            typestr = ' ({})'.format(option.typ.__class__.__name__)
+            typestr = ' ({})'.format(option.typ.get_name())
             lines.append("# {}{}:".format(optname, typestr))
 
             try:
@@ -430,7 +509,7 @@ class ConfigManager(QObject):
                 continue
             for descline in desc.splitlines():
                 lines += wrapper.wrap(descline)
-            valid_values = option.typ.valid_values
+            valid_values = option.typ.get_valid_values()
             if valid_values is not None:
                 if valid_values.descriptions:
                     for val in valid_values:
@@ -521,7 +600,15 @@ class ConfigManager(QObject):
                 k = self.RENAMED_OPTIONS[sectname, k]
             if (sectname, k) in self.CHANGED_OPTIONS:
                 func = self.CHANGED_OPTIONS[(sectname, k)]
-                v = func(v)
+                new_v = func(v)
+                if new_v is None:
+                    exc = configexc.ValidationError(
+                        v, "Could not automatically migrate the given value")
+                    exc.section = sectname
+                    exc.option = k
+                    raise exc
+
+                v = new_v
 
             try:
                 self.set('conf', sectname, k, v, validate=False)
@@ -572,15 +659,11 @@ class ConfigManager(QObject):
     def read(self, configdir, fname, relaxed=False):
         """Read the config from the given directory/file."""
         self._fname = fname
-        if configdir is None:
-            self._configdir = None
-            self._initialized = True
-        else:
-            self._configdir = configdir
-            parser = ini.ReadConfigParser(configdir, fname)
-            self._from_cp(parser, relaxed)
-            self._initialized = True
-            self._validate_all()
+        self._configdir = configdir
+        parser = ini.ReadConfigParser(configdir, fname)
+        self._from_cp(parser, relaxed)
+        self._initialized = True
+        self._validate_all()
 
     def items(self, sectname, raw=True):
         """Get a list of (optname, value) tuples for a section.
@@ -689,12 +772,12 @@ class ConfigManager(QObject):
             raise cmdexc.CommandError("set: {} - {}".format(
                 e.__class__.__name__, e))
 
-    @cmdutils.register(name='set', instance='config')
+    @cmdutils.register(name='set', instance='config', star_args_optional=True)
     @cmdutils.argument('section_', completion=Completion.section)
     @cmdutils.argument('option', completion=Completion.option)
-    @cmdutils.argument('value', completion=Completion.value)
+    @cmdutils.argument('values', completion=Completion.value)
     @cmdutils.argument('win_id', win_id=True)
-    def set_command(self, win_id, section_=None, option=None, value=None,
+    def set_command(self, win_id, section_=None, option=None, *values,
                     temp=False, print_=False):
         """Set an option.
 
@@ -710,7 +793,7 @@ class ConfigManager(QObject):
         Args:
             section_: The section where the option is in.
             option: The name of the option.
-            value: The value to set.
+            values: The value to set, or the values to cycle through.
             temp: Set value temporarily.
             print_: Print the value after setting.
         """
@@ -729,27 +812,45 @@ class ConfigManager(QObject):
             print_ = True
         else:
             with self._handle_config_error():
-                if option.endswith('!') and option != '!' and value is None:
+                if option.endswith('!') and option != '!' and not values:
+                    # Handle inversion as special cases of the cycle code path
                     option = option[:-1]
                     val = self.get(section_, option)
-                    layer = 'temp' if temp else 'conf'
                     if isinstance(val, bool):
-                        self.set(layer, section_, option, str(not val).lower())
+                        values = ['false', 'true']
                     else:
                         raise cmdexc.CommandError(
                             "set: Attempted inversion of non-boolean value.")
-                elif value is not None:
-                    layer = 'temp' if temp else 'conf'
-                    self.set(layer, section_, option, value)
-                else:
+                elif not values:
                     raise cmdexc.CommandError("set: The following arguments "
                                               "are required: value")
 
+                layer = 'temp' if temp else 'conf'
+                self._set_next(layer, section_, option, values)
+
         if print_:
             with self._handle_config_error():
                 val = self.get(section_, option, transformed=False)
-            message.info(win_id, "{} {} = {}".format(
-                section_, option, val), immediately=True)
+            message.info("{} {} = {}".format(section_, option, val))
+
+    def _set_next(self, layer, section_, option, values):
+        """Set the next value out of a list of values."""
+        if len(values) == 1:
+            # If we have only one value, just set it directly (avoid
+            # breaking stuff like aliases or other pseudo-settings)
+            self.set(layer, section_, option, values[0])
+        else:
+            # Otherwise, use the next valid value from values, or the
+            # first if the current value does not appear in the list
+            assert len(values) > 1
+            val = self.get(section_, option, transformed=False)
+            try:
+                idx = values.index(str(val))
+                idx = (idx + 1) % len(values)
+                value = values[idx]
+            except ValueError:
+                value = values[0]
+            self.set(layer, section_, option, value)
 
     def set(self, layer, sectname, optname, value, validate=True):
         """Set an option.
@@ -771,11 +872,23 @@ class ConfigManager(QObject):
         except KeyError:
             raise configexc.NoSectionError(sectname)
         mapping = {key: val.value() for key, val in sect.values.items()}
+
         if validate:
             interpolated = self._interpolation.before_get(
                 self, sectname, optname, value, mapping)
+            try:
+                allowed_backends = sect.values[optname].backends
+            except KeyError:
+                # Will be handled later in .setv()
+                pass
+            else:
+                backend = usertypes.arg2backend[objreg.get('args').backend]
+                if (allowed_backends is not None and
+                        backend not in allowed_backends):
+                    raise configexc.BackendError(backend)
         else:
             interpolated = None
+
         try:
             sect.setv(layer, optname, value, interpolated)
         except KeyError:
@@ -786,8 +899,6 @@ class ConfigManager(QObject):
 
     def save(self):
         """Save the config file."""
-        if self._configdir is None:
-            return
         configfile = os.path.join(self._configdir, self._fname)
         log.destroy.debug("Saving config to {}".format(configfile))
         with qtutils.savefile_open(configfile) as f:
